/**************************************************************************************************
   USB_Driver
   This module is used for communications with the user through a custom PC program.
**************************************************************************************************/
#include "USB_Driver.h"
#include "Buffers.h"
#include "LED_Driver.h"
#include "built_in.h"


// Variables
char readbuff[64];
char writebuff[64];

USB_TxStruct USB_Queue;        // Transmit buffer FIFO and input/output pointers

char TempBuf[] = "01234567";      // Test string

/**************************************************************************************************
   This is the USB interrupt handler
**************************************************************************************************/

void USBInterrupt() iv IVT_INT_OTG_FS ics ICS_AUTO {
     USB_Interrupt_Proc();
}

/**************************************************************************************************
   Initialization
**************************************************************************************************/
void USB_Init(void) {
     //USBIE_bit = 0;
     //IPC11bits.USBIP = 7;

     HID_Enable(&readbuff,&writebuff);
     USB_TxQueueInit();
     //NVIC_IntEnable(IVT_INT_OTG_FS);  // Enable USB device interrupt
}

/**************************************************************************************************
   This routine initializes the transmit buffer FIFO in and out pointers
**************************************************************************************************/
void USB_TxQueueInit(void) {

     USB_Queue.in = 0;
     USB_Queue.out = 0;
}

/**************************************************************************************************
   This routine makes an entry into the transmit FIFO and updates the in pointer to the next
   available location.
**************************************************************************************************/
void USB_PutQueue(char *msg) {

     USB_Queue.q[USB_Queue.in++] = msg;
     if (USB_Queue.in > (USB_Queue_Size-1))
         USB_Queue.in = 0;
}

/**************************************************************************************************
   This routine checks the transmit buffer FIFO and, if it is not empty, sends the buffer to USB.
   If the buffer was successfully sent, the out pointer is updated to the next available location.
**************************************************************************************************/

// Call this function in main loop
void USB_CheckOut(void) {

     if (USB_Queue.in != USB_Queue.out) {          // Look for a pointer in the FIFO

          // This is a blocking function
          if (HID_Write(USB_Queue.q[USB_Queue.out],64) != 0) {
              USB_Queue.out++;
              if (USB_Queue.out > (USB_Queue_Size-1))  // If past the end of array
                  USB_Queue.out = 0;                   // set pointer to start of array
          }
     }
}

/**************************************************************************************************
   This routine checks USB for a receive message then decodes the first character for a command.
**************************************************************************************************/

// Call this function in main loop
void USB_CheckIn(void) {
     char *BufNum;
     unsigned CmdDat;

     if (HID_Read() != 0) {
          switch (readbuff[0]) {

               case 'a':  // Diagnostic test of USB
                   BufNum = GetBuffer();
                   sprintf(BufNum,"Command 'a' received");
                   USB_PutQueue(BufNum);
                   break;
               case 'b':  // Another diagnostic test
                   USB_PutQueue("Command 'b' received");
                   break;
               case 'c':  // This only works because the high order nibble is ignored by MAX6951
                   DisplayAll(&TempBuf);
                   break;
               case 'd':  // Display hex character A in digit 0
                   SetDigit(0,0x0A,0); // Try changing digits and hex character
                   break;
               case 'e':  // Blank digit 0
                   SetDigit(0,0x00,0); // The digit code has to be 0
                   SetDecodeReg(0xFE); // then bit 0 of Decode Reg can be turned off
                   break;
               case 'f':  // Step intensity lower
                   LowerIntensity(); // The steps will decrease until at 0
                   break;
               case 'g':  // Step intensity higher
                   RaiseIntensity(); // The steps will increase until at 15
                   break;
               case 'h':  // Put display in low power mode
                   Shutdown(); // The display goes
                   break;
               case 'i':  // Set Normal display mode
                   Normal();
                   break;
               case 'j':  // Does not work
                   BlinkSlow();
                   break;
               case 'k':  // Does not work
                   BlinkFast();
                   break;
               case 'l':
                   BlinkOff();
                   break;
               case 'm':  // Turn all segments of the display on for test
                   DispTest(0x01);
                   break;
               case 'n':  // End test
                   DispTest(0x00);
                   break;
               case 'o':

                   break;
               case 'p':

                   break;
               case 'q':

                   break;
               case 'r':

                   break;
               case 's':

                   break;
               case 't':  // Blink enable (should be 1 second on/off)
                   Test(0x03);
                   break;
               case 'u':  // Bit D2 does nothing
                   Test(0x07);
                   break;
               case 'v':  // Bit D4 does nothing
                   Test(0x09);
                   break;
               case 'w':  // Increment through all possible codes of the Configuration register
                   Increment();
                   break;
               case 'x':  // Force normal mode
                   Test(0x01);
                   break;
               case 'y':

                   break;
               case 'z':  // Force a system reset
                   SystemReset();
                   break;

               // Etc.

               default:  // Catch all other unimplemented commands
                   BufNum = GetBuffer();  // Get a free buffer
                   (void)sprintf(BufNum,"Unknown command: %c\r\n",readbuff[0]);
                   USB_PutQueue(BufNum);
                   break;
          }
     }
}



/**************************************************************************************************
  EOF
**************************************************************************************************/